"""Run a OpenFOAM command"""

import sys
import string
from os import path

from FoamThread import FoamThread
from PyFoam.RunDictionary.SolutionDirectory import SolutionDirectory
from PyFoam.Execution.ParallelExecution import LAMMachine

class BasicRunner:
    """Base class for the running of commands

    When the command is run the output is copied to a LogFile and
    (optionally) standard-out

    The argument list assumes for the first three elements the
    OpenFOAM-convention:

    <cmd> <dir> <case>

    The directory name for outputs is therefor created from <dir> and
    <case>
    
    Provides some handle-methods that are to be overloaded for
    additional functionality"""
    
    def __init__(self,argv=None,silent=False,logname="PyFoam",lam=None):
        """@param argv: list with the tokens that are the command line
        if not set the standard command line is used
        @param silent: if True no output is sent to stdout
        @param logname: name of the logfile
        @param lam: Information about a parallel run
        @type lam: PyFoam.Execution.ParallelExecution.LAMMachine"""
        
        if argv==None:
            self.argv=sys.argv[1:]
        else:
            self.argv=argv
        self.dir=path.join(self.argv[1],self.argv[2])

        self.silent=silent
        self.lam=lam
        if self.lam!=None:
            nr=str(lam.cpuNr())
            self.argv=["mpirun","-np",nr]+self.argv[0:3]+["-parallel"]+self.argv[3:]
        self.cmd=string.join(self.argv," ")

        self.logFile=path.join(self.dir,logname+".logfile")
        
        self.fatalError=False
        self.warnings=0
        self.started=False
        
        self.run=FoamThread(self.cmd)


    def start(self):
        """starts the command and stays with it till the end"""
        
        self.started=True
        fh=open(self.logFile,"w")

        self.startHandle()
        
        self.run.start()

        while self.run.check():
            self.run.read()
            line=self.run.getLine()

            if not self.silent:
                print line

            if line.find("FOAM FATAL ERROR")>=0 or line.find("FOAM FATAL IO ERROR")>=0:
                self.fatalError=True

            if line.find("FOAM Warning")>=0:
                self.warnings+=1
                
            self.lineHandle(line)
            
            fh.write(line+"\n")
            fh.flush()

        self.stopHandle()
        fh.close()

    def runOK(self):
        """checks whether the run was successful"""
        if self.started:
            return not self.fatalError
        else:
            return False
        
    def startHandle(self):
        """to be called before the program is started"""
        pass
    
    def stopHandle(self):
        """called after the program has stopped"""
        pass
    
    def lineHandle(self,line):
        """called every time a new line is read"""
        pass
    
    def logName(self):
        """Get the name of the logfiles"""
        return self.logFile

    def getSolutionDirectory(self,archive=None):
        """@return: The directory of the case
        @rtype: PyFoam.RunDictionary.SolutionDirectory
        @param archive: Name of the directory for archiving results"""

        return SolutionDirectory(self.dir,archive=archive)
