"""Thread wrappers for OpenFOAM"""
from threading import Thread,Lock,Timer
from popen2 import Popen4
from time import time
from resource import getrusage,getpagesize,RUSAGE_SELF,RUSAGE_CHILDREN
from os import uname

from PyFoam.Basics.LineReader import LineReader

def getLinuxMem(thrd):
    """Reads the Memory usage of a thread on a linux-System

    @param thrd: the thread object in question"""

    #    print "Timer called"
    
    if not thrd.isLinux or thrd.threadPid<0:
        return

    mem=0

    try:
        t=open('/proc/%d/status' % thrd.threadPid)
        v=t.read()
        t.close()
        #        f=open('/tmp/test%dstatus' % thrd.threadPid,'w')
        #        f.write(v)
        #        f.close()
        
        i=v.index('VmRSS')
        tmp=v[i:].split()
        if len(tmp)>=3:
            mem=long(tmp[1])
            if tmp[2].lower()=='kb':
                mem*=1024
            elif tmp[2].lower()=='mb':
                mem*=1024*1024
            else:
                mem=-1
    except Exception,e:
        print "Getting LinuxMem:",e
        mem=-1
        
    if mem>thrd.linuxMaxMem:
        #        print "Setting Memory to: ",mem
        thrd.linuxMaxMem=mem
        
    #    print "Restarting Timer"
    thrd.timer=Timer(thrd.timerTime,getLinuxMem,args=[thrd])
    thrd.timer.start()
    
class FoamThread(Thread):
    """Thread running an OpenFOAM command

    The output of the command can be accessed in a thread-safe manner,
    line by line

    Designed to be used by the BasicRunner-class"""
    
    def __init__(self,cmdline):
        """cmdline - Command line of the OpenFOAM command"""
        Thread.__init__(self)
        self.cmdline=cmdline
        self.output=None
        self.reader=LineReader()

        self.isLinux=False
        self.isDarwin=False
        self.threadPid=-1
        self.who=RUSAGE_CHILDREN
        
        if uname()[0]=="Linux":
            self.isLinux=True
            self.linuxMaxMem=0
        elif uname()[0]=="Darwin":
            self.isDarwin=True
            #            self.who=RUSAGE_SELF
            
        self.timerTime=5.
        
        self.stateLock=Lock()
        self.setState(False)

        self.status=None

        self.lineLock=Lock()
        self.line=""

        self.resStart=None
        self.resEnd=None

        self.timeStart=None
        self.timeEnd=None
        
        self.stateLock.acquire()

    def run(self):
        """start the command"""
        # print "Starting ",self.cmdline
        self.resStart=getrusage(self.who)
        self.timeStart=time()
        
        run=Popen4(self.cmdline)
        self.output=run.fromchild
        self.threadPid=run.pid
        if self.isLinux:
            #            print "Starting Timer"
            self.timer=Timer(0.1*self.timerTime,getLinuxMem,args=[self])
            self.timer.start()
            
        self.hasSomethingToSay=True
        self.stateLock.release()

        try:
            # print "Waiting",time()
            self.status=run.wait()
            # Python 2.3 on Mac OS X never seems to reach this point
            # print "After wait",time()
            # print "Status:",self.status
        except OSError,e:
            print "Exeption caught:",e
            
        if self.isLinux:
            self.timer.cancel()
        self.threadPid=-1

        self.resEnd=getrusage(self.who)
        self.timeEnd=time()
        # print "End:",self.timeEnd
        # print "Returned",self.status

    def read(self):
        """read another line from the output"""
        self.setState(self.reader.read(self.output))
        self.lineLock.acquire()
        self.line=self.reader.line
        self.lineLock.release()

    def getLine(self):
        """gets the last line from the output"""
        self.lineLock.acquire()
        val=self.line
        self.lineLock.release()

        return val
    
    def setState(self,state):
        """sets the state of the thread (is there any more output)"""
        self.stateLock.acquire()
        self.hasSomethingToSay=state
        #        print "Set: ",state
        self.stateLock.release()
        
    def check(self):
        """@return: False if there is no more output of the command"""
        self.stateLock.acquire()
        state=self.hasSomethingToSay
        #        print "Get: ",state
        self.stateLock.release()

        return state 
    
    def cpuTime(self):
        """@return: number of seconds CPU-Time used"""
        return self.cpuUserTime()+self.cpuSystemTime()
    
    def cpuUserTime(self):
        """@return: number of seconds CPU-Time used in user mode"""
        if self.resEnd==None: # and self.isDarwin:
            # Mac OS X needs this (Ubuntu too?)
            self.resEnd=getrusage(self.who)
        if self.resStart==None or self.resEnd==None:
            return 0
        else:
            return self.resEnd.ru_utime-self.resStart.ru_utime

    def cpuSystemTime(self):
        """@return: number of seconds CPU-Time used in system mode"""
        if self.resEnd==None: #  and self.isDarwin:
            # Mac OS X needs this (Ubuntu too?)
            self.resEnd=getrusage(self.who)
        if self.resStart==None or self.resEnd==None:
            return 0
        else:
            return self.resEnd.ru_stime-self.resStart.ru_stime

    def usedMemory(self):
        """@return: maximum resident set size in MegaByte"""
        scale=1024.*1024.
        if self.isLinux:
            return self.linuxMaxMem/scale
        
        if self.resStart==None or self.resEnd==None:
            return 0.
        else:
            return getpagesize()*(self.resEnd.ru_maxrss-self.resStart.ru_maxrss)/scale

    def wallTime(self):
        """@return: the wall-clock-time needed by the process"""
        if self.timeEnd==None: #  and self.isDarwin:
            # Mac OS X needs this (Ubuntu too?)
            self.timeEnd=time()
            
        #        print "Wall:",self.timeEnd,self.timeStart
        if self.timeStart==None or self.timeEnd==None:
            return 0
        else:
            return self.timeEnd-self.timeStart
        
